//////////////////////////////////////////////////////////////////////////
//
// Physics replay for e08007
//
//////////////////////////////////////////////////////////////////////////
//	
//  Based on Jin Huang's online replay package
//////////////////////////////////////////////////////////////////////////


#include "def.h"

using namespace std;

void replay_phys_all(Int_t runnumber=0,Int_t all=0)
{
  
  //Add L-HRS
  THaApparatus* HRSL = new THaHRS("L","Left arm HRS");
  gHaApps->Add( HRSL);
  HRSL->AddDetector( new THaFpp("fpp","Focal Plane Polarimeters"));
  HRSL->AddDetector( new THaCherenkov("cer", "Gas Cherenkov counter" ));
  HRSL->AddDetector( new THaShower("prl1", "Pre-shower pion rej." ));
  HRSL->AddDetector( new THaShower("prl2", "Show pion rej." ));
   
  //Add BigBite

  THaBigBite* pB = new THaBigBite("BB","BigBite"); 
  THaBBTotalShower* ts = new THaBBTotalShower("ts","BigBite total shower");
  pB->AddDetector( new THaScintillator( "s", "scintillator" ));
  pB->AddDetector( new THaScintillator( "sum", "scintillator"));
  pB->AddDetector( new THaScintillator( "psum", "scintillator"));
  pB->AddDetector(ts);
  gHaApps->Add(pB);

  //Decode

  THaApparatus* DECDAT = new THaDecData("DL","Misc. Decoder Data");
  gHaApps->Add( DECDAT );

  // Add left arm scalers
  THaScalerGroup* LeftScalers = new THaScalerGroup("Left");
  LeftScalers->Init();
  gHaScalers->Add(LeftScalers);
  
  //Enable Scaler
  //analyzer->EnableScalers();
  
  //Helicity
  //THaHelicityDet *hleft = new THaHelicityDet("HL","Beam helicity L-arm");
  //hleft->SetState (1, 8, -1, 0, 0);  // G0 mode; 8 window delay; sign -1;
  // left arm; no redund
  //hleft->SetROC (0, 11, 0, 3, 0, 4); // "Left arm" is ROC 11
  //BEAM->AddDetector(hleft);
  
  // Add an unrastered, ideally positioned and directed electron beam
  THaApparatus* BEAM = new THaIdealBeam("Beam","Simple ideal beamline");
  gHaApps->Add( BEAM );  
  THaApparatus* RBEAM = new THaRasteredBeam("rb","Rastered beam");
   gHaApps->Add( RBEAM );
  //   THaApparatus* UBEAM = new THaUnRasteredBeam("urb","Unrastered beam");
  //   gHaApps->Add( UBEAM );
  
  //Add L-arm Helicity
  BEAM->AddDetector( new THaADCHelicity("adchel.L","Beam helicity L-arm") );
  BEAM->AddDetector( new THaADCHelicity("adchel2.L","Beam helicity-2 L-arm") );
  BEAM->AddDetector( new THaG0Helicity("g0hel.L","Left arm G0 helicity") );	
  
  
  // Prepare the physics modules to use
  
  // Watch the electron kinematics
  Double_t amu = 931.494*1.e-3;  // amu to GeV
  Double_t mass_he3 = 2.809413; // in GeV
  Double_t mass_C12 = 12.0*amu; // AMU to GeV
  Double_t mass_tg  = mass_C12;  // chosen for Carbon elastic calibration runs
  Double_t mass_p = 0.93827; //proton mass in GeV
  
  // Plain, uncorrected Electron kinematics
  THaPhysicsModule* EK_l = new THaElectronKine("EK_L",
  					       "Electron kinematics in HRS-L",
  					       "L",mass_p);

  gHaPhysics->Add( EK_l );

  gHaPhysics->Add( new THaGoldenTrack("L.gold","Golden track for LHRS", "L") );
  gHaPhysics->Add( new GammaModule("GM","Gamma Reconstruction from LHRS","L"));
  gHaPhysics->Add( new SAProtonModule("SAP","Proton in HRSL","L"));


  // Use the reaction vertex to correct for an extended target
  // Cheating for now, and assuming a perfect, unrastered electron beam
  THaPhysicsModule *Rpt_l = new THaReactionPoint("ReactPt_L","Reaction vertex for Left",
						 "L","Beam");
  gHaPhysics->Add( Rpt_l );


  // Correct for using an Extended target

  THaPhysicsModule* TgC_l = new THaExtTarCor("ExTgtCor_L",
					     "Corrected for extended target, HRS-L",
					     "L","ReactPt_L");
  gHaPhysics->Add( TgC_l );
  
  
  // Finally, the CORRECTED Electron kinematics (note change of "spectrometer")
  THaPhysicsModule* EKxc_l = new THaElectronKine("EKxc_L","Electron kinematics in HRS-L",
						 "ExTgtCor_L",mass_p);
  gHaPhysics->Add( EKxc_l );
  
  //Normalization analysis ( DT, charge ... )
  
  THaNormAna* norm = new THaNormAna("N","Normalization Analysis");
  
  Int_t evint[10];  // event intervals of stable beam
  evint[0]=4;       // Number of intervals
  evint[1]=450;
  evint[2]=2000;    // beam stable in [450,2000]
  evint[3]=4000;    // [4000,8000].
  evint[4]=8000;
  norm->DoBcmCalib();
  norm->SetEventInt(evint);
  gHaPhysics->Add( norm );
   
  ReplayCore(
	    runnumber,            //run #
	    all,                  //-1=replay all;0=ask for a number
	    -1,                   //defaut replay event num
	    "%s/e08007_%d.root", //output file format
	    "replay_phys_all.odef",       //out define
	    "replay_phys.cdef",        //cut define
	    true                       //enable scaler
	    );
}
